*==============================================================================
* Function:			NEXTID
* Purpose:			Get the next available value for the specified field
* Author:			Doug Hennig
* Copyright:		(c) 1995 Stonefield Systems Group Inc.
* Last Revision:	04/21/98
* Parameters:		tcAlias - the alias of the table of interest (optional:
*						if it wasn't specified, the current table is used)
*					tcField - the name of the field to assign (optional: if
*						it wasn't specified, the primary key is used)
* Returns:			the next available key value (either numeric or left-
*						justified string, depending on the field type), or a
*						null string if there was a problem
* Environment in:	the table of interest is open
*					the NEXTID table is either open or is available to be
*						opened
* Environment out:	if NEXTID was open, it is positioned to the appropriate
*						record for the field and table
*					if there was no record for the field and table in NEXTID,
*						one is created with a starting value of 1
*					the KEY field of the appropriate record in NEXTID is
*						incremented
*==============================================================================

#include F:\TOOLS\VFP\LIBRARY\INCLUDES\LIBRARY.H
lparameters tcAlias, ;
	tcField
local lcAlias, ;
	luReturn ,;
	lcDatabase, ;
	llView, ;
	lcField, ;
	lcCurrDBC, ;
	lcTable, ;
	lcTag, ;
	lcFile, ;
	llUsed, ;
	lnCurrReprocess, ;
	lcMax, ;
	laID[1], ;
	liKey

* If the alias wasn't passed, use the currently selected table. Give an
* error and return if there is no table or the table isn't open.

lcAlias  = iif(type('tcAlias') <> 'C' or empty(tcAlias), alias(), ;
	upper(tcAlias))
luReturn = 0
luReturn = .NULL.
do case
	case empty(lcAlias)
		= messagebox(ccERR_NO_TABLE_SELECTED)
		return luReturn
	case not used(lcAlias)
		= messagebox(strtran(ccERR_TABLE_NOT_OPEN, ccMSG_INSERT1, lcAlias))
		return luReturn
endcase
lcDatabase = cursorgetprop('Database',   lcAlias)
llView     = cursorgetprop('SourceType', lcAlias) <> 3
do case

* If the field was passed, use it.

	case type('tcField') = 'C' and not empty(tcField)
		lcField = upper(tcField)

* If the alias is a view or a free table, give an error since we don't know
* which field to update.

	case llView or empty(lcDatabase)
*** PROPER MESSAGE HERE
		= messagebox('You must specify a field when NEXTID is called for ' + ;
			iif(llView, 'a view.', 'a free table.'))
		return luReturn

* If the field wasn't passed, find the primary key for the table.

	otherwise
		lcCurrDBC = dbc()
		if not empty(lcDatabase) and set('DATABASE') <> lcDatabase
			set database to (lcDatabase)
		endif not empty(lcDatabase) ...
		lcTable = cursorgetprop('SourceName', lcAlias)
		lcTag   = dbgetprop(lcTable, 'Table', 'PrimaryKey')
		set database to &lcCurrDBC
		if empty(lcTag)
			= messagebox(ccERR_NO_FIELD)
			return luReturn
		else
			lcFile  = dbf(lcAlias)
			lcFile  = substr(lcFile, rat('\', lcFile) + 1)
			lcFile  = left(lcFile, rat('.', lcFile) - 1)
			lcField = key(tagno(lcTag, lcFile, lcAlias), lcAlias)
			if ',' $ lcField or '+' $ lcField
				= messagebox(strtran(ccERR_PRIM_COMPOUND, ccMSG_INSERT1, lcAlias))
				return luReturn
			endif ',' $ lcField ...
		endif empty(lcTag)
endcase

* Open the NEXTID table (if necessary).

llUsed = used('NEXTID')
if llUsed
	select NEXTID
	set order to NEXTID
else
	select 0
	use NEXTID order NEXTID again shared
endif llSYS_USED

* Try to find a record for the field. If there isn't one, create it.

lnCurrReprocess = set('REPROCESS')
set reprocess to 10 seconds
seek padr(lcAlias, len(TABLE)) + padr(lcField, len(FIELD))
if not found()
	lcMax = iif(type(lcAlias + '.' + lcField) = 'C', 'val(' + lcField + ')', ;
		lcField)
	select max(&lcMax) from (lcAlias) into array laID
	liKey = iif(_tally > 0, laID[1], 0)
	insert into NEXTID (TABLE, FIELD, KEY) values (lcAlias, lcField, liKey)
endif not found()

* If we can lock the record, increment the KEY value (rolling over to 1 if
* necessary) and assign it to luReturn.

if rlock()
	liKey = iif(KEY >= 2^32/2 - 1, 1, KEY + 1)
	replace KEY with liKey
	luReturn = iif(type(lcAlias + '.' + lcField) = 'C', ltrim(str(liKey)), ;
		liKey)
	unlock
endif rlock()

* Cleanup and return.

if not llUsed
	use in NEXTID
endif not llUsed
set reprocess to lnCurrReprocess
return luReturn
